function [z]= crudetrack2(z0,t,gamma,g,f)
% Crude path tracking algorithm for the homotopy
%   h(z,t) = gamma*t*( g(z) ) + (1-t)*( f(z) )
% where g(z) and f(z) are polynomials in one variable given as row vectors
% of coefficients in the usual Matlab fashion
npath=length(z0);
M=length(t);
z=zeros(npath,M);
% prepare derivatives of g and f w.r.t z
gz = polyderiv(g);
fz = polyderiv(f);
% Do the path tracking
for kpath=1:npath  % loop over the start points
    zz = z0(kpath);
    z(kpath,1)=zz;
    for kstep=1:(M-1) % loop through the time steps
        t0=t(kstep);
        t1=t(kstep+1);
        zpredict = euler(zz,t0,t1,gamma,g,f,gz,fz);       %prediction
        zz       = newton(zpredict,t1,gamma,g,f,gz,fz); %correction
        z(kpath,kstep+1)=zz;
    end
    % an extra call to newton to refine the endpoint
    z(kpath,M) = newton(zz,t(M),gamma,g,f,gz,fz);
end

function [pz] = polyderiv(p)
% derivative of a polynomial in one variable
d = length(p)-1; % degree
pz = p(1:d).*[d:-1:1];

function [z1]=euler(z0,t0,t1,gamma,g,f,gz,fz)
% Euler prediction step
ht = gamma*polyval(g,z0) - polyval(f,z0);
hz = gamma*t0*polyval(gz,z0) + (1-t0)*polyval(fz,z0);
z1 = z0 - (t1-t0) * ht / hz;

function [z1]=newton(z0,t,gamma,g,f,gz,fz)
% Newton correction step.  Use 5 iterations.
z=z0;
for ii=1:5
    h = gamma*t*polyval(g,z) + (1-t)*polyval(f,z);
    hz = gamma*t*polyval(gz,z) + (1-t)*polyval(fz,z);
    z = z - h/hz;
end
z1=z;