function [tpath,zpath,success,zerror,Herror]= goodtrack(z0,tspan,H,Ht,Hz)
% goodtrack:  path tracker with variable step size
%   for multivariate homotopy function h(z,t): C^n X C --> C^n
%
% Inputs: 
%   z0 = start value of z, a column vector
%   tspan = [t_start t_final],  usually this is [1 0]
%   H  = pointer to homotopy function h(z,t)
%   Hz = pointer to function returning the Jacobian matrix dh/dz
%   Ht = pointer to function returning dh/dt
%   H,Ht,Hz all take the arguments (z,t)
% Outputs:
%   t,z = points along the homotopy path
%   success = 1 if accurate result is obtained
%   zerror  = Newton residual at end
%   Herror  = magnitude of h(z,t_final)
%
% We make some hard-coded choices of control parameters.
% In a better implementation, these are made available to the outside
% for adjustment by the user, or for automatic adaptation.
stepstart=.1;      % size of first tracking step as fraction of the whole tspan
stepmin  =1e-6;    % minimum step size allowed before failure is declared
maxit    =3;       % number of Newton corrector iterations allowed per step
maxnfe   =1500;    % total # of function evaluations allowed (guards against runaway paths)
epstiny  =1e-12;   % used for detecting zero length tangent vector
%
%endgame parameters
t_endzone= 0.1;    % final fraction of tspan considered to be the "endgame" 
epsbig   =1e-4;    % tracking accuracy to be maintained in initial tracking until t=t_endgame
epssmall =1e-6;    % tracking accuracy to be maintained in endgame t=[t_endgame,tstop]

% initializations
goodstep=0; nfe=0;

% set up regarding t
t=tspan(1);
tgoal=tspan(2);
stepsize=stepstart*abs(tgoal-t);
tdirection=sign(tgoal-t);
t_end = t_endzone*abs(tgoal-t);

% set up for z
z=z0;

% store the first point
kk=1;
tpath(kk)=t; zpath(:,kk)=z0;

% tracking loop.
epstrack=epsbig;
while abs(t-tgoal)>1e-12,
   % Step t.  Do not step past the goal.
   stepsize=min(abs(t-tgoal),stepsize);
   tnew=t+stepsize*tdirection; 
   % PREDICT 
   znew = predict(z,t,tnew,Hz,Ht,epstiny);
   % CORRECT
   [znew,errsize,success,nfe2]=correct(znew,tnew,H,Hz,epstrack,maxit);
   nfe=nfe+nfe2;
   % STEP LENGTH ADJUSTMENT
   if success 
      % step was good.  Update and store
      z=znew; t=tnew;
      kk=kk+1; tpath(kk)=t; zpath(:,kk)=z;
      % If 5 successive good steps, double stepsize.
      goodstep=goodstep+1; 
      if goodstep==5, stepsize=2*stepsize; goodstep=0; end;
      % Check if we're in the end zone and if so, tighten tolerance
      if abs(t-tgoal) <= t_end, epstrack=epssmall; end
   else,
      % step failed.
      goodstep=0;
      % cut stepsize
      stepsize=stepsize/2.;
      % check if we should quit
      success = stepsize>stepmin & nfe<maxnfe;
      if ~success, 
         zerror = Inf;
         Herror = max(abs(feval(H,z,tgoal)));
         return;  % return with last point on path and success=0
      end;
   end;
end;
% ENDPOINT REFINEMENT
% We've reached the goal t, refine the answer with Newton
[z,zerror] = correct(z,tgoal,H,Hz,1e-12,10);
Herror = max(abs(feval(H,z,tgoal)));

function [znew] = predict(z,t,tnew,Hz,Ht,epstiny)
% Euler step prediction
tangent = -feval(Hz,z,t)\feval(Ht,z,t);  % -Hz^(-1) Ht
% detect zero-length tangent error (should not happen for correct
% homotopies, but check just in case)
if max(abs(tangent))<epstiny, 
  disp(['Path stalled at t=' num2str(t)])
  error('Tangent vector is zero.')
end;
znew = z + tangent*(tnew-t);

function [z,errsize,success,nfe]=correct(z0,t,H,Hz,epstrack,maxit)
% Newton method corrector
z=z0;
for iter=1:maxit
    dz = -feval(Hz,z,t)\feval(H,z,t);
    z = z + dz;
    errsize = max(abs(dz));
    if errsize<epstrack,
        success=1; nfe=iter;
        return;
    end
end
errsize=Inf; success=0; nfe=maxit;
