% BURMEST  
% Solve the four-bar body-guidance problem for five given locations
% This is the classical "Burmester five-position problem"
%
% Formulate as 2-homogeneous in 4 variables having 6 paths.
% See Eq.9.6.46 of Sommese & Wampler.  Also, Exercise 9.8
%
% We expect two roots at infinity.
% That is, the problem has only 4 roots on (C^*)^4
%
% 

disp('BURMEST -- four-bar body guidance')
disp('  Using a 2-homogeneous formulation in 4 variables as in')
disp('  Eq.9.6.46 of Sommese & Wampler.')
disp('  Implemented via tableau format (i.e., fully expanded),')
disp('  so very inefficient.  The problem can be reduced to a')
disp('  quartic in one variable, if desired.')
disp(' ')
if ~exist('THETA','var') | ~exist('P','var') | ~exist('PBAR','var')
	disp('Usage: set up three 1x5 arrays, THETA, P, and PBAR, before')
	disp('  calling BURMEST.')
    disp('  THETA is the coupler rotation angle in RADIANS.')
    disp('  (P(i),PBAR(i)) are isotropic coordinates for the ')
    disp('    coupler position. For a "real" problem, set P(i) as')
    disp('    as the position in the complex plane and use: ')
    disp('    >> PBAR = conj(P)')
    if exist('THETA','var'), disp('Make sure THETA is in upper case.'); end
    if exist('p','var'), disp('Make sure P is in upper case.'); end
    if exist('pbar','var'), disp('Make sure PBAR is in upper case.'); end
    error('THETA or P or PBAR missing.')
end

if length(THETA)~=5
    error('THETA must have 5 entries.')
end
if length(P)~=5
    error('P must have 5 entries.')
end
if length(PBAR)~=5
    error('PBAR must have 5 entries.')
end

% compute the exponential representation of the angles
THETAexp = exp(1i*THETA);
THETAbar = 1./THETAexp;
% the rows of CCMat are the coeffients on 
%   1, x, a, xbar, abar, x*abar, xbar*a
%   respectively
CCMat = [
    P.*PBAR
    THETAexp.*PBAR
    PBAR
    P.*THETAbar
    P
    THETAexp
    THETAbar
];
% subtract the first from each of the rest
CoefMat = CCMat * [ -ones(1,4); eye(4) ];

% Make the tableau
% monomial list with an end-of-polynomial marker
monos = [
     0 0 0 0
    eye(4)
     1 0 0 1
     0 1 1 0
    -1 0 0 0
];
tableau = [
    reshape([CoefMat; 0 0 0 0],32,1) [monos;monos;monos;monos]
];
[tableau,varscale,eqnscale]=scalepol(tableau);

% define the two-homogeneous structure
uv    = [ 1 1 0 0 1 0 ];
uvbar = [ 0 0 1 1 0 1 ];
HomStruct=[
    uv
    uvbar
];

% solve it
mhomtab

% dehomgenize answer & display it
xx = diag([varscale;1;1])*dehomog(xsoln,1e-8,HomStruct);
disp('Variables are ordered as [x,a,xbar,abar,w,wbar],')
disp('where w & wbar are homogenizing coordinates.')
disp('The solutions are stored in xx=')
disp(xx)
