% FCNGEN
% Solve the four-bar function generation problem for five given angle pairs
%
% Formulate as 2-homogeneous system in 4 variables
% having root count 6.
% The homotopy has 4 variables and 6 paths.
% See Eq.9.6.44 of Sommese & Wampler.  Also, Exercise 9.7
%
% We expect two roots at infinity and a trivial root.
% That is, the problem has only 3 roots on (C^*)^4
%
% 

disp('FCNGEN -- four-bar function generation')
disp('  Using a 2-homogeneous formulation in 4 variables as in')
disp('  Eq.9.6.44 of Sommese & Wampler.')
disp('  Implemented via tableau format (i.e., fully expanded),')
disp('  so very inefficient.  The problem can be reduced to a')
disp('  cubic in one variable, if desired.')
disp(' ')
if ~exist('PHI','var') | ~exist('PSI','var')
	disp('Usage: set up two 1x5 arrays, PHI and PSI, before')
	disp('  calling FCNGEN. These are the input & output')
	disp('  angles in RADIANS.')
    if exist('phi','var'), disp('Make sure PHI is in upper case.'); end
    if exist('psi','var'), disp('Make sure PSI is in upper case.'); end
    error('PHI or PSI missing.')
end

if length(PHI)~=5
    error('PHI must have 5 entries.')
end
if length(PSI)~=5
    error('PSI must have 5 entries.')
end

% compute the exponential representation of the angles
phiexp = exp(1i*PHI);
psiexp = exp(1i*PSI);

% the rows of CCMat are the coeffients on u, v, ubar, vbar, u*vbar, ubar*v,
%   respectively
CCMat = [
    -phiexp
    psiexp
    -1./phiexp
    1./psiexp
    -phiexp./psiexp
    -psiexp./phiexp
];
% subtract the first from each of the rest
CoefMat = CCMat * [ -ones(1,4); eye(4) ];
% For each equation, rescale largest coefficient to 1
CoefMat = CoefMat * diag(1./max(abs(CoefMat)));

% Make the tableau
% monomial list with an end-of-polynomial marker
monos = [
    eye(4)
     1 0 0 1
     0 1 1 0
    -1 0 0 0
];
tableau = [
    reshape([CoefMat; 0 0 0 0],28,1) [monos;monos;monos;monos]
];
[tableau,varscale,eqnscale]=scalepol(tableau);

% define the homogenization 
uv    = [ 1 1 0 0 1 0 ];
uvbar = [ 0 0 1 1 0 1 ];
HomStruct = [
    uv
    uvbar
];

% solve it
mhomtab

% dehomgenize answer & display it
xx = diag([varscale;1;1])*dehomog(xsoln,1e-8,HomStruct);
disp('Variables are ordered as [u,v,ubar,vbar,w,wbar],')
disp('where w & wbar are homogenizing coordinates.')
disp('The solutions are stored in xx=')
disp(xx)
