% FCNGEN1HOM
% Solve the four-bar function generation problem for five given angle pairs
%
% Homogenize 1-homogeneously but solve with linear-product
% structure having the same root count, 6, as the natural
% 2-homogeneous treatment of the problem.
% The homotopy has 4 variables and 6 paths.
% See Eq.9.6.44 of Sommese & Wampler.  Also, Exercise 9.7
%
% We expect two roots at infinity and a trivial root.
% That is, the problem has only 3 roots on (C^*)^4
%
% 

disp('FCNGEN1HOM -- four-bar function generation')
disp('  Using a linear-product formulation in 4 variables as in')
disp('  Eq.9.6.44 of Sommese & Wampler.')
disp('  We 1-homgenize the equations to capture solutions going')
disp('  to infinity, but use linear products to get the same ')
disp('  root count as a 2-homogeneous formulation. ')
disp('  Implemented via tableau format (i.e., fully expanded),')
disp('  so very inefficient.  The problem can be reduced to a')
disp('  cubic in one variable, if desired.')
disp(' ')
if ~exist('PHI','var') | ~exist('PSI','var')
	disp('Usage: set up two 1x5 arrays, PHI and PSI, before')
	disp('  calling FCNGEN. These are the input & output')
	disp('  angles in RADIANS.')
    if exist('phi','var'), disp('Make sure PHI is in upper case.'); end
    if exist('psi','var'), disp('Make sure PSI is in upper case.'); end
    error('PHI or PSI missing.')
end

if length(PHI)~=5
    error('PHI must have 5 entries.')
end
if length(PSI)~=5
    error('PSI must have 5 entries.')
end

% compute the exponential representation of the angles
phiexp = exp(1i*PHI);
psiexp = exp(1i*PSI);

% the rows of CCMat are the coeffients on u, v, ubar, vbar, u*vbar, ubar*v,
%   respectively
CCMat = [
    -phiexp
    psiexp
    -1./phiexp
    1./psiexp
    -phiexp./psiexp
    -psiexp./phiexp
];
% subtract the first from each of the rest
CoefMat = CCMat * [ -ones(1,4); eye(4) ];
% For each equation, rescale largest coefficient to 1
CoefMat = CoefMat * diag(1./max(abs(CoefMat)));

% Make the tableau
% monomial list with an end-of-polynomial marker
monos = [
    eye(4)
     1 0 0 1
     0 1 1 0
    -1 0 0 0
];
tableau = [
    reshape([CoefMat; 0 0 0 0],28,1) [monos;monos;monos;monos]
];
[tableau,varscale,eqnscale]=scalepol(tableau);

% define the homogenization and the linear-product structure
HomStruct = []; % take the default to 1-hom
uv    = [ 1 1 0 0 1 ];
uvbar = [ 0 0 1 1 1 ];
LPDdegrees=[
    2 2 2 2
];
LPDstruct=[
    uv; uvbar;
    uv; uvbar;
    uv; uvbar;
    uv; uvbar;
];

% solve it
lpdtab

% dehomgenize answer & display it
xx = diag([varscale;1])*dehomog(xsoln,1e-8,HomStruct);
disp('Variables are ordered as [u,v,ubar,vbar,w],')
disp('where w is the homogenizing coordinate.')
disp('The solutions are stored in xx=')
disp(xx)
