function [f,fx,fp]=sixrev1f(x,sixrev_par)
% Two homogeneous version of six-revolute problem
%  using Eqs. 9.4.30 -- 9.4.32 in Sommese & Wampler
% Homogeneous groups are <1,z2,z4> and <1,z3,z5>
% This version is for parameter homotopy.
% There are 23 parameters, passed in as sixrev_par
%
% Two of the equations are linear, so for efficiency we could reduce the
% number of variables, but for simplicity, we do not do so here.

% rename parameters for readability
cos_alpha = sixrev_par(1:5,1);
a_over_sin_alpha = sixrev_par(6:10,1);
d_length = sixrev_par(11:14,1);
phat = sixrev_par(15:17,1);
z1 = sixrev_par(18:20,1);  % z1 & z6 should be unit vectors in physically meaningful use
z6 = sixrev_par(21:23,1);

% rename variables
z2 = x(1:3,1);
z3 = x(4:6,1);
z4 = x(7:9,1);
z5 = x(10:12,1);

% homogenizing coordinates
u = x(13);
v = x(14);
uu=u^2;
uv= u*v;
vv=v^2;

% the function
f = [
    (z2.'*z2 - uu)*0.5
    (z3.'*z3 - vv)*0.5
    (z4.'*z4 - uu)*0.5
    (z5.'*z5 - vv)*0.5
    (z1.'*z2 - u *cos_alpha(1))
    (z2.'*z3 - uv*cos_alpha(2))
    (z3.'*z4 - uv*cos_alpha(3))
    (z4.'*z5 - uv*cos_alpha(4))
    (z5.'*z6 - v *cos_alpha(5))
    (   (a_over_sin_alpha(1)*v)*cross(z1,z2) +...
        (a_over_sin_alpha(2)  )*cross(z2,z3) + (d_length(1)*v)*z2 +...
        (a_over_sin_alpha(3)  )*cross(z3,z4) + (d_length(2)*u)*z3 +...        
        (a_over_sin_alpha(4)  )*cross(z4,z5) + (d_length(3)*v)*z4 +...
        (a_over_sin_alpha(5)*u)*cross(z5,z6) + (d_length(4)*u)*z5 +...
        (-phat)*uv)
];
% Jacobian for x
% fx is sparse, so initialize it to zero & then fill in.
fx = zeros(12,14);
% unit vector equations
fx(1, 1:3 ) = z2.'; fx(1,13) = -u;
fx(2, 4:6 ) = z3.'; fx(2,14) = -v;
fx(3, 7:9 ) = z4.'; fx(3,13) = -u;
fx(4,10:12) = z5.'; fx(4,14) = -v;
% twist angle equations
fx(5, 1:3 ) = z1.'; fx(5,13) = -cos_alpha(1);
fx(6, 1:3 ) = z3.'; fx(6, 4:6 ) = z2.'; fx(6,13) = -v*cos_alpha(2); fx(6,14) = -u*cos_alpha(2);
fx(7, 4:6 ) = z4.'; fx(7, 7:9 ) = z3.'; fx(7,13) = -v*cos_alpha(3); fx(7,14) = -u*cos_alpha(3);
fx(8, 7:9 ) = z5.'; fx(8,10:12) = z4.'; fx(8,13) = -v*cos_alpha(4); fx(8,14) = -u*cos_alpha(4);
fx(9,10:12) = z6.'; fx(9,14) = -cos_alpha(5);
% position closure
fx(10:12,:) = [
    ( crossmat( (a_over_sin_alpha(1)*v)*z1 ) - crossmat( (a_over_sin_alpha(2)  )*z3 ) + (d_length(1)*v)*eye(3) ), ...
    ( crossmat( (a_over_sin_alpha(2)  )*z2 ) - crossmat( (a_over_sin_alpha(3)  )*z4 ) + (d_length(2)*u)*eye(3) ), ...
    ( crossmat( (a_over_sin_alpha(3)  )*z3 ) - crossmat( (a_over_sin_alpha(4)  )*z5 ) + (d_length(3)*v)*eye(3) ), ...
    ( crossmat( (a_over_sin_alpha(4)  )*z4 ) - crossmat( (a_over_sin_alpha(5)*u)*z6 ) + (d_length(4)*u)*eye(3) ), ...
    ( d_length(2)*z3 + a_over_sin_alpha(5)*cross(z5,z6) + d_length(4)*z5 - phat*v ),...
    ( a_over_sin_alpha(1)*cross(z1,z2) + d_length(1)*z2 + d_length(3)*z4 - phat*u )
];
% Jacobian for parameters
% even more sparse, so let's use the Matlab sparse matrix form
fp = sparse([],[],[],12,23,100);
% derivatives for cos_alpha(i)
fp(5,1) = -u;
fp(6,2) = -uv;
fp(7,3) = -uv;
fp(8,4) = -uv;
fp(9,5) = -v;
% derivatives for a_over_sin_alpha(i)
fp(10:12,6:10) = [ v*cross(z1,z2) cross(z2,z3) cross(z3,z4) cross(z4,z5) u*cross(z5,z6) ];
% derivatives for d_length(i)
fp(10:12,11:14)= [ v*z2 u*z3 v*z4 u*z5 ];
% derivatives for phat
fp(10,15) = -uv; fp(11,16) = -uv; fp(12,17) = -uv;
% derivatives for z1
fp(5,18:20) = z2.';
fp(10:12,18:20) = crossmat((-a_over_sin_alpha(1)*v)*z2);
% derivatives for z6
fp(9,21:23) = z5.';
fp(10:12,21:23) = crossmat(( a_over_sin_alpha(5)*u)*z5);

function [c]=cross(a,b)
% cross product
% Matlab has a cross function, but it is slow due to checking.
c = [
    a(2)*b(3)-a(3)*b(2)
    a(3)*b(1)-a(1)*b(3)
    a(1)*b(2)-a(2)*b(1)
];

function [px]=crossmat(p)
% vector cross-product
% crossmat(p)*q = cross(p,q)
px=[0 -p(3) p(2); 
    p(3) 0 -p(1);
   -p(2) p(1) 0];
