% Solve a Stewart-Gough forward kinematics problem by linear-product homotopy

% generate random, complex Stewart-Gough platform of general type
a = rand(3,6) + 1i*rand(3,6);
b = rand(3,6) + 1i*rand(3,6);
legsq = rand(1,6) + 1i*rand(1,6);

% precompute corresponding A,B matrices so that forward kinematic
%  equations are just the quadrics x.'*ABi*x = 0
global AB1 AB2 AB3 AB4 AB5 AB6
[AB1]=stewab(a(:,1),b(:,1),legsq(1));
[AB2]=stewab(a(:,2),b(:,2),legsq(2));
[AB3]=stewab(a(:,3),b(:,3),legsq(3));
[AB4]=stewab(a(:,4),b(:,4),legsq(4));
[AB5]=stewab(a(:,5),b(:,5),legsq(5));
[AB6]=stewab(a(:,6),b(:,6),legsq(6));

% Do a reduction: subtract leg 1 from leg i to cancel the gg' term
AB2 = AB2-AB1;
AB3 = AB3-AB1;
AB4 = AB4-AB1;
AB5 = AB5-AB1;
AB6 = AB6-AB1;

% ProjTransf is an inhomogeneous linear slice to pick out a unique point in C^8 on each
%  projective line
global ProjTransf
ProjTransf = rand(1,8) + 1i*rand(1,8);

% Define the linear-product structure of the start system
global nvar degrees FFUN
nvar=7;  % we're working on CP^7
degrees=[2 2 2 2 2 2 2]; % all quadratics
% Note, Astruct=ones(14,8) would give a total-degree start system with 128 roots
% Instead, we use 
%       <e,g>^2   for leg 1, 
%       <e,g>x<e> for legs 2,...,6, and 
%       <e>x<g>   for the Study quadric
eg=[1 1 1 1 1 1 1 1];
e =[1 1 1 1 0 0 0 0];
g =[0 0 0 0 1 1 1 1];
Astruct=[
 eg;eg;
 eg;e;
 eg;e;
 eg;e;
 eg;e;
 eg;e;
 e;g; 
];
FFUN='sgfun';
% lpdsolve forms the start system and runs the homotopy
disp('The start system has 30 singular solutions of the form e=0.')
disp('Routine LPDSOLVE warns about these and rejects them.')
disp('This is perfectly OK.')
disp('After pausing 5 seconds, the computation will begin.')
for i=1:5, pause(1); disp('.'); end
lpdsolve
% the answers are in xsoln and stats

% Just in case, rerun picks out any end points that aren't accurate
%  and re-traces the homotopy path with tighter tolerances.
rerun

% Examine the results to see how many nondegenerate solutions there are
% Non-degenerate ones will be in array: xnotd
sgfilter

% draw a histogram to show the degenerate vs nondegenerate judged by the
%   maximum magnitude of the elements of e=x(1:4)
emax = log10(max(abs(xsoln(1:4,:))));
hist(emax)
title('Histogram of Largest Element of e=x(1:4)')
xlabel('Log10(max(abs(e)))')
ylabel('# of Solutions')

disp('If you would like to save the non-degenerate solutions, enter a file name.')
fname = input('Enter filename or leave blank for no save: ','s');
if length(fname)>0,
  p0=[reshape(a,18,1);reshape(b,18,1);reshape(legsq,6,1)];
  eval(['save ' fname ' xnotd p0']);
end