% Solve general Stewart-Gough platform using a total-degree homotopy
disp('Solving a general Stewart-Gough platform using a total-degree homotopy.')
disp('The platform will be randomly generated (complex).')
disp('There will be 128 homotopy paths.')

% generate random, complex Stewart-Gough platform of general type
a = rand(3,6) + 1i*rand(3,6);
b = rand(3,6) + 1i*rand(3,6);
legsq = rand(1,6) + 1i*rand(1,6);

% precompute corresponding A,B matrices so that forward kinematic
%  equations are just the quadrics x.'*ABi*x = 0
global AB1 AB2 AB3 AB4 AB5 AB6
[AB1]=stewab(a(:,1),b(:,1),legsq(1));
[AB2]=stewab(a(:,2),b(:,2),legsq(2));
[AB3]=stewab(a(:,3),b(:,3),legsq(3));
[AB4]=stewab(a(:,4),b(:,4),legsq(4));
[AB5]=stewab(a(:,5),b(:,5),legsq(5));
[AB6]=stewab(a(:,6),b(:,6),legsq(6));

% ProjTransf is an inhomogeneous linear slice to pick out a unique point in C^8 on each
%  projective line
% gamma is a constant for the homotopy
global ProjTransf gamma
ProjTransf = rand(1,8) + 1i*rand(1,8);
gamma = exp(3i*rand(1,1));

% start system is x(1)^2=x(i)^2
% the following generates the 128 starting solutions 
xstart=ones(8,128);
for ii=1:7
   rr = rem(0:127,2^ii); halfcycle=(2^ii-1)/2.;
   xstart(ii+1,:) = (rr>halfcycle) - (rr<halfcycle);
end

% Endgamer runs the homotopy with an endgame to compute singular roots accurately
htopyset;
[xsoln,stats] = endgamer(xstart,'htotdeg');

% Just in case, rerun picks out any end points that aren't accurate
%  and re-traces the homotopy path with tighter tolerances.
startpoint=xstart; hfun='htotdeg';
rerun

% Examine the results to see how many nondegenerate solutions there are
% Non-degenerate ones will be in array: xnotd
sgfilter

% draw a histogram to show the degenerate vs nondegenerate judged by the
%   maximum magnitude of the elements of e=x(1:4)
emax = log10(max(abs(xsoln(1:4,:))));
hist(emax)
title('Histogram of Largest Element of e=x(1:4)')
xlabel('Log10(max(abs(e)))')
ylabel('# of Solutions')

disp('If you would like to save the non-degenerate solutions, enter a file name.')
fname = input('Enter filename or leave blank for no save: ','s');
if length(fname)>0,
  p0=[reshape(a,18,1);reshape(b,18,1);reshape(legsq,6,1)];
  eval(['save ' fname ' xnotd p0']);
end