% casctab
% A script for computing witness supersets of a system given in tableau format
% using a total degree cascade homotopy
%
% On input, 
%
% Output is stored in cell arrays Wsuper{} and Lslice{}, indexed by
% codimension, 1,...,rank f.
%
% We assume the system is not the zero polynomial, so we skip codimension 0
%
% This script assumes the tableau is in the workspace in matrix "tableau"
% which has
%   1) coefficients in first column
%   2) exponents of variables x1,...,xn in columns 2,...,(n+1)
%   3) end of polynomials marked with a -1 in column 2
%   4) we assume the system is NOT homogeneous, and we
%      one-homogenize it.

% decode the tableau and homogenize it
global nummono coeflist exponent
[NSYSpol,NSYSvar,SYSdegrees,nummono1,coeflist1,exponent1]=ftabprep(tableau);
% sort polynomials into descending degrees
[SYSdegrees,nummono,coeflist,exponent]=ftabsort(NSYSpol,SYSdegrees,nummono1,coeflist1,exponent1);
clear nummono1 coeflist1 exponent1 

epsNonsoln = 1e-8;

global FSYS FSQRD   
global nvar degrees 
global RandSup LSup
FSYS = 'ftabcall';  % pass to wsupfcn via global

% Square up the system via randomization & intrinsic slice
% Prepare randomization.  New # of eqs = rank 
% The polynomials are in order of descending degrees. 
% Use [I Lambda] form to avoid inflating the total degree.
% get rank of the system
rnk = ranksys(FSYS,NSYSvar); 
RandSup = [eye(rnk) crand(rnk,NSYSpol-rnk)];
% Intrinsic slice. The "+1" is for homogenizing variables.
%   map homogeneous coordinate straight through
LSup = [
    crand(NSYSvar,rnk+1) 
    zeros(1,rnk)    1
];

% total degree product structure
nvarF = rnk;
degreeF = SYSdegrees(1:nvarF);
homvar=nvarF+1;  % this is the homogenizing coordinate
LPDstructF= ones(sum(degreeF),nvarF+1);
HomStruct=[];   % default to 1-hom

% run the cascade
epsNonsoln = 1e-8;
eps_inf  =1e-8;

[Wsuper,Lslice,Stats,Nonsoln,NonsolnStats,Degen,DegenStats,Bad,BadStats] = ...
    cascade('wsupfun',nvarF,degreeF,LPDstructF,HomStruct,homvar,epsNonsoln,eps_inf);

% un-do the squaring
for kk=1:rnk
    Wsuper{kk} = LSup*Wsuper{kk};
    Nonsoln{kk}= LSup*Nonsoln{kk};
    Degen{kk}  = LSup*Degen{kk};
    Bad{kk}    = LSup*Bad{kk};
end
if ~isempty(Wsuper{rnk}) & rnk<NSYSpol
    % Wsuper at lowest dimension could contain nonsolutions
    nn = size(Wsuper{rnk},2);
    ferr = zeros(1,nn);
    for ii=1:nn
        ferr(ii) = max(abs(ftabcall(Wsuper{rnk}(:,ii))));
    end
    nonsoln = ferr>epsNonsoln;
    Nonsoln{rnk} = Wsuper{rnk}(:,nonsoln);
    NonsolnStats{rnk} = Stats{rnk}(:,nonsoln);
    Wsuper{rnk}  = Wsuper{rnk}(:,~nonsoln);
    Stats{rnk}   = Stats{rnk}(:,~nonsoln);
end