function [y]=dehomog(x,eps0,HomStruct,homvar)
% y=dehomog(x,eps0) or y=dehomg(x,eps0,HomStruct,homvar)
% INPUTS
%   x         = a matrix whose columns are points in projective space
%           or in a cross product of projective spaces
%   eps0      = zero tolerance. 
%   HomStruct = (optional) mhom-by-(nvar+mhom) matrix of 1's and zeros
%          Element (i,j)=1 indicates variable j is in group i.
%          Each variable must be in exactly one group.
%          If missing or empty, default to 1-homogeneous in all variables.
%   homvar    = (optional) 1-by-mhom list identifying the homogenizing variable for
%          each group.  If missing or empty, assume last variable in the
%          group.
% OUTPUT
%   y   = de-homogenized version of x 
%
% Behavior:
%   for each column i:
%         if x(hom(j),i) has magnitude greater than eps, we dehomogenize group j; 
%         else the group is considered to be at infinity and we normalize by dividing
%              the column by its largest entry.
%   This means that on output, one element in each group in each column is set to 1.
n=size(x,1);
if nargin<3, 
    % default to 1-homogeneous and last coordinate
    HomStruct=ones(1,n); mhom=1; homvar=n;
elseif nargin<4,
    if isempty(HomStruct)
        HomStruct=ones(1,n);
    end
	mhom=size(HomStruct,1);
    for ii=1:mhom
        members=find(HomStruct(ii,:));
        homvar(ii) = members(length(members));
    end
else
    mhom=size(HomStruct,1);
end;
% loop through, dehomogenizing each group
y=zeros(size(x));
for ii=1:mhom
    members=find(HomStruct(ii,:));
	finite = find( abs(x(homvar(ii),:))>eps0 );
    % w = rescaling = biggest element if at infinity or homvar if finite.
	w = max(x(members,:));
	w(finite) = x(homvar(ii),finite);
	y(members,:) = x(members,:) * diag(1./w);
end