function [nvarE,degreeE,AstructE,Lslice,Lambda]=embedset(nvarF,degreeF,AstructF)
% function [nvarE,degreeE,AstructE,Lslice,Lambda]=embedset(nvarF,degreeF,AstructF)
%   set up embedding for cascade algorithm for computing witness superset
%
%  Inputs describe the system F to be solved:
%	nvarF   = # of variables
%	degreeF = row of degrees of equations
%	AstructF= linear product structure (see lpdsolve)
%
%  Outputs are for the embedded system E 
%   See Chapter 14, Sommese & Wampler for details
%   This algorithm is based on the paper Sommese & Verschelde, "Numerical homotopies to
%	compute generic points on positive dimensional algebraic sets,"
%	J. of Complexity, 16:572-602, 2000.
%	However, eliminate the slack variables to keep nvarE=nvarF
%	AstructE = as in F but modified by inclusion of slicing linears
%	Lslice = coefficient matrix for the linear slice
%	Lambda = coefficient matrix for adding slice equations to F
%  These work with function evaluator [f,fx,ft]=embed(x,t) and CASCADE.M, which
%       use lpdsolve to get roots dimension nvarF and then cascades down to all 
%	lower dimensions.
%  The embedded system is
%	E(x,T) = F(x) + Lambda * diag(T) * (Lslice * [x;1]) = 0
nvarE=nvarF; nvar=nvarF;
degreeE=degreeF;
Lslice = rand(nvar,nvar+1) + 1i*rand(nvar,nvar+1);
Lambda = rand(nvar,nvar)   + 1i*rand(nvar,nvar);
% AstructE must have linear term for each variable and constant term
[nfac,ncol]=size(AstructF);
if ncol ~= nvar+1, error('Expecting AstructF to have nvar+1 columns.'); end
if nfac ~= sum(degreeF), error('Expecting AstructF to have sum(degreeF) rows.'); end;
if length(degreeF) ~= nvar, error('Expecting degreeF to have length nvar.'); end;
% copy AstructF but make sure constant term is present
AstructE = [AstructF(:,1:nvar) ones(nfac,1)];
k=0;
for ii=1:nvar
   d = degreeE(ii);
   % check for missing linear factors in LPD of ii-th eqn
   AA = AstructE( (k+1):(k+d), : );
   missing = ~max(AA~=0 );
   % if any are missing, add them to the factor that is already densest
   %  this is a heuristic for getting a low count.  Always gives a valid
   %  structure, but maybe not always the lowest count.
   [y,jj]=max(sum(AA'));
   AstructE( (k+jj),: ) = ( AA(jj,:)|missing );
   k = k+d;
end
