% Homotopy continuation with fractional power-series endgame
% function [xsoln,stats,xendgame,successprep,tprep]=endgamer(startpoint,hfun,allownewpatch)
%
% Perform continuation given startpoints and a homotopy function.
%   Use the power-series endgame
% INPUTS:
%   startpoint = matrix of start points, listed column-wise
%   hfun       = string name of homotopy function.  It must be a
%                homogeneous function of the form [h,dhx,dht]=hfun(x,t)
%                where
%                   hfun(startpoint(:,i),1)=0
%                and
%                   hfun(x,0) is the final problem to be solved
%   allownewpatch= 1, if hfun is homogeneous and we wish to allow a new
%                   projective patch to be chosen at beginning of endgame
%                0, if hfun not homogeneous, or we don't wish to allow new
%                   new patch.
%                if missing or empty, default to 0.
%     NOTE: for Linear Product Decomposition homotopies, see LPDsolve
%           for parameter homotopies, see PARsolve
% OUTPUTS:
%   xsoln  = matrix of solutions, listed column-wise
%   stats  = i-th column is 
%           [t, errsize, fcn_err, cycle #, cond. #, # of fcn evals] 
%           for solution i
%   xendgame = solution path at start of endgame (use for checking for 
%              path crossing).
%     NOTE: xsoln and xendgame are in projective space.  If the original
%     problem variables are in Euclidean space, you will need to dehomogenize.
%     (See function dehomog.m)
%   successprep = success vector for endprep, in case we want to re-run just
%       the endgame
%   tprep  = t at the end of endprep
%
% METHOD:
%  See A.P. Morgan, A.J. Sommese, and C.W. Wampler, "A power series method
%  for computing singular solutions to nonlinear analytic systems,"
%  Numerische Mathematik, 63:391--409, 1992.
%
%  Collects points and tangents on incoming homotopy path.  Predicts next
%  point on path using fractional power series and, after correction, tests 
%  which cycle number 1,2,...,MaxCycle gave the best prediction.  This 
%  cycle number used to predict the value at the origin.  Advance until
%  successive predictions agree to high accuracy or path cannot be tracked.
%
%  This implementation uses cubic prediction on the incoming path and then
%  predicts across s=0 to pick up a symmetric sample allowing a 7th order
%  fit of the final endpoint (via a cubic fit of the symmetrically averaged
%  sample points).
function [xsoln,stats,xendgame,successprep,tprep]=endgamer(startpoint,hfun,allownewpatch)
global ProjTransf
% apply the Projective Transform to the startpoints
[startpoint]=patchdo(startpoint,ProjTransf);
% track from t=1 to t=t_endgame
disp('Tracking paths to t=t_endgame...')
[xendgame,successprep,tprep,nfe]=endprep(hfun,startpoint);
% Call endpower to play the power-series endgame
if nargin<3, allownewpatch=0; end
disp('Begin power-series endgame...')
[xsoln,stats]=endpower(hfun,xendgame,successprep,tprep,nfe,allownewpatch);
