function [tt,xx_end,xxdt_end,c_end,status,success,nfe]=...
                 endmove(hfun,tt,xx_end,xxdt_end,c_end,status)
% ENDMOVE.m  =  advance homotopy path in endgame.
% 
% new tgoal determined from tt(1) and global parameter tratio
%   as tgoal = tt(1)*tratio
% 
% Predict to tgoal with various trial cycle numbers.
% Correct at tgoal using prediction of current cycle # estimate.
% If not successful, fall back on standard tracker.
%
% Once we get to tgoal, determine which cycle number made best prediction.
% Return that value as new estimate of cycle number.
% Update the endpoint data.
% 
% Calling sequence:
%  [tt,xx_end,xxdt_end,c_end,status,success,nfe]=...
%                 endmove(hfun,tt,xx_end,xxdt_end,c_end,status)
%
% INPUTS
%  hfun     = string name of homotopy function
%  tt       = t value of last two samples
%  xx_end   = last two points on homotopy path
%  xxdt_end = derivatives at xx_end
%  c_end    = last two cycle number estimates
%  status   = number of samples at current cycle number (1,2,3,or 4)
% OUTPUTS
%  tt,xx_end,xxdt_end,c_end,status = same as above, updated with new sample
%  success  = 1 if a new sample was obtained
%  nfe      = number of function evaluations used
%
% GLOBALS 
%  tratio maxit epssmall CycleMax epssample

global tratio maxit epssmall CycleMax
global epssample

tgoal = tt(1)*tratio;
% predict to tgoal for each cycle number
c = [1:CycleMax];
if status==1
    % linear prediction adjusted for cycle number
    xpredict = xx_end(:,1)*ones(1,CycleMax) + xxdt_end(:,1)*(tt(1)*c.*(tratio.^(1./c)-1));
else
    % cubic prediction
    xpredict = cubicpre(tgoal,tt,xx_end,xxdt_end,c);
end

% Try to jump all the way.  If unsuccessful, fall back on default tracker
cycle=c_end(1);
x = xpredict(:,cycle);
[xnew,errsize,success,nfe]=newton2(hfun,x,tgoal,epssmall,maxit,1);
if success,
    % refine the point to high tolerance
    % allow 10 iterations, but don't persist if error increases
    [x,es,ss,nfe2] = newton2(hfun,xnew,tgoal,epssample,10,0);
else
	% default tracker
    tanlen = sum(abs(xxdt_end(:,1)));
    %stepsize = 0.7*(tt(1)-tgoal)*tanlen;
    stepsize = 0.51*(tt(1)-tgoal); % go just past half way
    %disp([tt(1) stepsize cycle]);
	[x,t,success,stepsize,errsize,nfe2]=...
                 tracker(hfun,xx_end(:,1),xxdt_end(:,1),tt(1),stepsize,tgoal,epssmall,cycle,inf);
end
nfe=nfe+nfe2;

if ~success, 
    % don't update, just return
    return; 
end

% determine which cycle number made best prediction
diff = x*ones(1,CycleMax) - xpredict;
[y,i]=sort(max(abs(diff)));
cycle=i(1);

% update endpoint information
tt = [tgoal tt(1)];
xx_end = [x xx_end(:,1)];
[h,dhx,dht]=feval(hfun,x,tgoal);
tangent = -dhx\dht;
xxdt_end = [tangent xxdt_end(:,1)];
c_end = [cycle c_end(1)];
if c_end(1)==c_end(2)
    % cycle number hasn't changed, we're building up a fit
    status = max(status,2); % have at least 2 points now
else
    % start building fresh with new cycle number
    status = 1;
end
