% [xsoln,stats]=endpower(hfun,xendgame,success,tfinal,nfe,allownewpatch)
%
% Fractional power-series endgame for homotopy continuation 
%
% INPUTS:
%   hfun       = string name of homotopy function.  It must be a
%                homogeneous function of the form [h,dhx,dht]=hfun(x,t)
%                where
%                   hfun(xendgame(:,i),t_endgame)=0
%                and
%                   hfun(x,0) is the final problem to be solved
%                and last coordinate of x is the homogeneous coordinate
%   xendgame = solution path at start of endgame, t=t_endgame (a global)
%   success  = row vector indicating which points in xendgame should be run
%              (Normally produced as output of endprep.)
%              if missing or empty, assume all points should be run.
%   tfinal   = last t in the endprep run.  
%              Copy this into stats for failed paths.
%   nfe      = number of function evaluations already used in endprep.
%              These are substracted from maxnfe (global) to get the
%              allowance for the endgame.
%              If missing, assumed as zero.
%   allownewpatch= 1, if hfun is homogeneous and we wish to allow a new
%                   projective patch to be chosen at beginning of endgame
%                0, if hfun not homogeneous, or we don't wish to allow new
%                   new patch.
%                if missing or empty, default to 0.
% OUTPUTS:
%   xsoln  = matrix of solutions, listed column-wise
%   stats  = i-th column is 
%           [t, errsize, fcn_err, cycle #, cond. #, # of fcn evals] 
%           for solution i
%     NOTE: xsoln and xendgame are in projective space.  If the original
%     problem variables are in Euclidean space, you will need to dehomogenize.
%     (See function dehomog.m)
%
% METHOD:
%  See A.P. Morgan, A.J. Sommese, and C.W. Wampler, "A power series method
%  for computing singular solutions to nonlinear analytic systems,"
%  Numerische Mathematik, 63:391--409, 1992.
%
%  Collects points and tangents on incoming homotopy path.  Predicts next
%  point on path using fractional power series and, after correction, tests 
%  which cycle number 1,2,...,MaxCycle gave the best prediction.  This 
%  cycle number used to predict the value at the origin.  Advance until
%  successive predictions agree to high accuracy or path cannot be tracked.
%
%  This implementation uses cubic prediction on the incoming path and then
%  predicts across s=0 to pick up a symmetric sample allowing a 7th order
%  fit of the final endpoint (via a cubic fit of the symmetrically averaged
%  sample points).

function [xsoln,stats]=endpower(hfun,xendgame,success,tfinal,nfe,allownewpatch);

% endgame control parameters
global epssmall t_endgame tstop tratio eps_end
global maxit maxnfe maxerrup allowjump
global verbose PulseTime

npath = size(xendgame,2);
if nargin<3, success=[]; end
if isempty(success), success=ones(1,npath); end;
if nargin<4, tfinal=[]; end
if isempty(nfe), tfinal=ones(1,npath); end;
if nargin<5, nfe=[]; end
if isempty(nfe), nfe=zeros(1,npath); end;
if nargin<6, allownewpatch=0; end

% timer for printing pulse message
t0clock = clock;

% formats for path completion messages
format compact; format short e;
% allocate space for outputs
xsoln=zeros(size(xendgame));
stats=zeros(6,size(xendgame,2));
tic; %start the clock for timing info

% Allocate space for the endgame as follows:
%   We keep a record of the last two points on the path:
%       tt,xx_end,xxdt_end,c_end  (t, x, dx/dt, cycle#)
tt=[0 0]; xx_end=zeros(size(xsoln,1),2); xxdt_end=xx_end; c_end=[1 1];
%   We also try to "jump across the origin" to get 2 symmetric points
%       xx_jump, xxdt_jump
xx_jump=xx_end; xxdt_jump=xxdt_end;
%   We predict the value at t=0, keeping the two most recent ones in
%       x0_est
x0_est=xx_end;
%   The difference in these two estimates the convergence.  The best
%   estimate by this measure is stored in: tbest,xbest
xbest = zeros(size(xsoln,1),1);
%   A variable "status" keeps track of where we are in this process, so
%   the order of prediction is linear, cubic or heptic depending on the
%   number of points in xx_end and xx_jump
%   We continue until convergence or failure, and report tbest,xbest
%
% MAIN LOOP: play endgame path-by-path
for path=1:npath
    if verbose, disp(['Endgame: Path ' num2str(path)]); end
    if ~success(path),
      xsoln(:,path)=xendgame(:,path); 
      stats(:,path)=[tfinal(path); inf; inf; NaN; NaN; nfe(path)];
      disp('skipped')
    else
      status=1;    % we're in the endgame with 1 point
      newsample=1; % initial point is the first sample
      % if allowed, reset projective patch as complex conjugate of x
      if allownewpatch
          x=patchnew(xendgame(:,path));
      else
          x=xendgame(:,path);
      end
      % Initialize the endgame with t, x, and tangent at current point
      t=t_endgame; 
      tt(1)=t; xx_end(:,1)=x; 
      [h,dhx,dht]=feval(hfun,x,t);
      xxdt_end(:,1)= -dhx\dht; 
      totnfe=nfe(path)+1;
      % Our best estimate of the endpoint is currently just the last point
      % on the path
      x0_est(:,1)=x;
      xbest=x; tbest=t; cbest=1; besterr=1;   % we're aiming to beat this
      converged=0; % flag for stopping when we've converged
      errup=0;     % counts successive failures to improve endpoint estimate
      % Endgame loop --- makes power-series predictions to t=0.
      %    There are 5 possible stopping conditions:
      %        1) successive predictions agree.  (best case)
      %        2) path cannot advance (i.e., stepsize becomes too small)
      %        3) too much computational cost (maxnfe exceeded)
      %        4) hit limit t<tstop.
      %        5) failed to improve endpoint estimate several times in a
      %        row
      while 1
        if ~newsample, break; end;  % success means we have new sample points
        % predict to t=0 using the latest path samples
        x0_est(:,2)=x0_est(:,1);
        x0_est(:,1)=predict0(tt,xx_end,xxdt_end,c_end,xx_jump,xxdt_jump,status);
        % compare last two estimates to measure convergence
        errsize=max(abs(x0_est*[1;-1]));
        % keep track of the best result so far
        if errsize<besterr,
           tbest=tt(1); xbest=x0_est(:,1); cbest=c_end(1);
           besterr=errsize; errup=0;
        else,
           errup=errup+1;
        end;
        if verbose>1 % print progress of each iteration if global verbose>1.
            disp([tt(1) errsize c_end(1) status errup]);
        end
        converged=errsize<eps_end;
        % check stopping conditions
        if converged | totnfe>maxnfe | tt(1)<tstop | errup==maxerrup, break; end;
        % get the next point on path and best cycle number for it
        [tt,xx_end,xxdt_end,c_end,status,newsample,nfemove]=...
                 endmove(hfun,tt,xx_end,xxdt_end,c_end,status);
        totnfe=totnfe+nfemove;
        % if cycle number has changed, reset the errup counter
        if c_end(1) ~= c_end(2), errup=0; end
        if newsample & status>1 & allowjump
            % update the jump samples
            xx_jump(:,2)=xx_jump(:,1); xxdt_jump(:,2)=xxdt_jump(:,1);
            [xx_jump(:,1),xxdt_jump(:,1),status,nfejump]=jumpsamp(hfun,tt,xx_end,xxdt_end,c_end,status,epssmall,maxit);
            totnfe=totnfe+nfejump;
        end
      end;
      % end of endgame for this path
      % report the best endpoint estimate
      % final function error and condition number
      [f,df]=feval(hfun,xbest,0);
      ferr=max(abs(f)); fcond=cond(df);
      % copy results to output
      xsoln(:,path)=xbest;
      stats(:,path)=[tbest;besterr;ferr;cbest;fcond;totnfe+1];
      if verbose, ttock=toc; disp(['  endgame time = ',num2str(ttock)]); end
    end; 
    % print "pulse message" to show life in a long run
    if ~verbose
        t1clock=clock;
        dtclock = etime(t1clock,t0clock);
        if dtclock>PulseTime
            t0clock=t1clock;
            disp(['Endgame: Path = ',num2str(path),', time = ',num2str(toc)])
        end
    end
end;
if ~verbose, ttock=toc; disp(['  endgame time = ',num2str(ttock)]); end
