% function [xendgame,success,tfinal,nfe]=endprep(hfun,startpoint)
%
% Perform continuation from startpoints to beginning of endgame.
% Note: you may use pathcros.m to check for path crossings before
%   calling endpower.m to play the power-series endgame.
%   
% INPUTS:
%   hfun       = string name of homotopy function.  It must be a
%                homogeneous function of the form [h,dhx,dht]=hfun(x,t)
%                where
%                   hfun(startpoint(:,i),1)=0
%                and
%                   hfun(x,0) is the final problem to be solved
%                This routine stops with solutions of
%                   h(xendgame,t_endgame)=0.
%   startpoint = matrix of start points, listed column-wise
%
% OUTPUTS:
%   xendgame   = solutions at t=t_endgame, listed columnwise 
%   success(i) = 1, if path i was successful, 0 else.
%   tfinal(i)  = last value of t for a successful step. This is t_endgame,
%                except for failed paths
%   nfe(i)     = number of function evaluations used for path i
%
% METHOD:
%  Simple predict-correct tracking using routine tracker.m
%  Tries first with tracking tolerance=epsbig (global) and if
%  unsuccessful, re-runs with epsbig/100.
%
% GLOBAL VARIABLES USED:
%  stepstart epsbig t_endgame stepmin maxit verbose PulseTime
%
function [xendgame,success,tfinal,nfe]=endprep(hfun,startpoint)

% endgame control parameters
global stepstart epsbig t_endgame stepmin maxit maxnfe verbose PulseTime
global ProjTransf

% allocate space
xendgame= zeros(size(startpoint));
npath   = size(startpoint,2);
success =zeros(1,npath);
tfinal  = ones(1,npath);
nfe     =zeros(1,npath);
tic; %start the clock for timing info
t0clock = clock;

% MAIN LOOP: track to t_endgame, path-by-path
for path=1:npath
    if verbose, disp(['Pre-endgame: Path ' num2str(path)]); end
    % initiate path tracking
    t=1.; x=startpoint(:,path); cycle=1;
    % make sure the point is on the current projective patch
    x=patchdo(x,ProjTransf);
    % check the startpoint
    [h,dhx,dht]=feval(hfun,x,t); 
    tangent=-dhx\dht;
    totnfe=1;
    if (max(abs(h')))>1e-8,
       disp(['Bad start values for path ' num2str(path)]);
       xp=x', hp=h'  % this line prints the point & homotopy value
       success(path)=0;    % don't start the endgame
    else,
        % track to t=t_endgame with loose tolerance
        [xnew,tnew,success(path),stepsize,errsize,nfetrack]= ...
                tracker(hfun,x,tangent,t,stepstart,t_endgame,epsbig,cycle,maxnfe);
        totnfe=totnfe+nfetrack;
        % if not successful, try again with tighter tolerance
        if success(path)
            x=xnew; t=tnew;
        else
            if verbose>1
                disp(['Path rerun with epsbig = ' num2str(epsbig/100)])
            end
            % rerun with tighter tolerance.
            [x,t,success(path),stepsize,errsize,nfetrack]= ...
                tracker(hfun,x,tangent,t,stepstart,t_endgame,epsbig/100,cycle,maxnfe);
            if ~success(path) & verbose
                disp(sprintf('Path Failure: path=%d, t=%e, stepsize=%e, nfe=%d',[path t stepsize nfetrack]))
            end
            totnfe=totnfe+nfetrack;
        end
    end;
    tfinal(path) = t;
    if success(path), 
        xendgame(:,path)=x;
    else
        xendgame(:,path)=NaN*x;
    end
    nfe(path) = totnfe;
    if verbose, 
        ttock=toc; disp(['  pre-endgame time = ',num2str(ttock)]); 
    else
        % print "pulse message" to show life in a long run
        t1clock=clock;
        dtclock = etime(t1clock,t0clock);
        if dtclock>PulseTime
            t0clock=t1clock;
            disp(['Pre-endgame: Path = ',num2str(path),', time = ',num2str(toc)])
        end
    end
end;
if verbose, else, ttock=toc; disp(['  pre-endgame time = ',num2str(ttock)]); end
disp(['Finished tracking to t_endgame with ',num2str(nnz(success)),' of ',num2str(npath),' paths successful.'])
