function [npol,nvar,degrees,nummono,coeflist,exponent]=ftabprep(tableau,HomStruct,ishom,LPDstruct,LPDdegrees)
% function [npol,nvar,degrees,nummono,coeflist,exponent]=ftabprep(tableau,HomStruct,ishom)
%
% Decodes and homogenizes a tableau-style specification of a polynomial
% system. (Or checks if already homgenized polynomial is OK.)
%
% INPUTS
%   tableau   = tableau matrix defining the system, with
%     1) coefficients in first column
%     2) exponents of variables x1,...,xn in columns 2,...,(n+1)
%     3) end of polynomials marked with a -1 in column 2
%   HomStruct = (optional) mhom-by-(nvar+mhom) matrix of 1's and zeros
%          Element (i,j)=1 indicates variable j is in group i.
%          Each variable must be in exactly one group.
%          If HomStruct missing or empty, default to 1-homogeneous in all variables.
%          If ishom==0 or missing, HomStruct must assign a homogenizing
%          coordinate for each group among the last mhom variables
%   ishom    = (optional) if ishom==1, tableau is already homogenized
%   LPDstruct = (optional) linear product-decomposition structure, having 
%          sum(LPDdegrees) number of rows and nvar+mhom columns.
%          There are LPDdegrees(i) rows for equation i, each representing
%          a linear factor, having 1 in column j if variable j appears
%          in the factor and 0 otherwise.  
%          If present, we use this to homogenize the system so that the
%          homotopy t*g+(1-t)*f will be homogeneous, where g is the LPD
%          start system and f is the system given by the tableau
%   LPDdegrees = if LPDstruct is present, this input must also be present
%          to allow LPDstruct to be decoded, as above.
% OUTPUTS
%   npol     = number of polynomials
%   nvar     = number of variables
%   degrees  = degree(:,i) is multi-degree of equation i
%   nummono  = nummono(i) is number of monomials in equation i
%   coeflist = coefficients extracted from first column
%   exponent = exponents of the monomials, homogenized
%
% Note: ordinarily, tableau is not homogenized, so this routine will
% homogenize it.  However, if ishom==1, it is assumed already homogenized.  
% Then, this routine checks that fact and warns if a discrepancy is found.

% first, determine what m-homogeneous structure is intended
ntab=size(tableau,2)-1;  
if nargin<2
    % default to 1-homogeneous 
    n=ntab+1;
    HomStruct=ones(1,n); ishom=0;
elseif nargin<3
    ishom=0;
end
if isempty(HomStruct)
    if ishom
        n=ntab;
    else
        n=ntab+1;
    end
    HomStruct=ones(1,n);
end
mhom=size(HomStruct,1);
% parse tableau into its individual polynomials
nvar = size(HomStruct,2)-mhom;
monos = tableau(:,2)>=0; % end of polynomial marked by negative exponent on x1
npol = nnz(~monos);
keep = find(monos);
coeflist = tableau(keep,1);
exponent = tableau(keep,2:(ntab+1))';
clear keep
% check that exponents are nonnegative integers
if any(any(floor(exponent)-exponent))
    error('All exponents must be integer')
end
if any(any(exponent<0))
    error('All exponents must be non-negative')
end
% If LPDstruct is given, find its multidegree
lpdmdeg=zeros(mhom,npol);
if nargin>3
    if (sum(LPDdegrees) ~= size(LPDstruct,1))
        error('LPDstruct and LPDdegrees not compatible. Sum of degrees must equal number of structure rows.')
    end
    A = (HomStruct*(LPDstruct.') ~= 0 );
    k=0;
    for ii=1:length(LPDdegrees)
        lpdmdeg(:,ii)=sum(A(:,k+[1:LPDdegrees(ii)]),2);
        k = k+LPDdegrees(ii);
    end
end
% count the monomials
eop = [0 find(~monos)']; % list of indices for end of polynomials
nummono = eop(2:(npol+1))-eop(1:npol)-1; % count the monomials
% Extract degrees and homogenize
degrees = zeros(mhom,npol);
if ishom
    % the tableau should be already homogenized
    % check it and compute degrees
    if ntab~=(nvar+mhom)
        error('Tableau and HomStruct not consistent, case ishom==1.')
    end
    jj=0;
    for ip=1:npol
        ii=jj+1; jj=jj+nummono(ip);
        mdegree = HomStruct*exponent(:,ii:jj);
        if any(max(mdegree,[],2)~=min(mdegree,[],2))
            error(['Polynomial ',num2str(ip),' not properly homogenized.']);
        else
            degrees(:,ip)=mdegree(:,1);
        end
        if nargin>3
            if any( degrees(:,ip) ~= lpdmdeg(:,ip) )
                error('Polynomial ',num2str(ip),' not homogenized compatible to LPDstruct.')
            end
        end
    end
else
    % we need to (multi)-homogenize
    if ntab~=nvar
        error('Tableau and HomStruct not consistent, case ishom==0.');
    end
    % allocate space for the homogenizing coordinates
    exponent = [exponent; zeros(mhom,size(exponent,2))];
    homvar=zeros(1,mhom);
    for ii=1:mhom
        members = find(HomStruct(ii,:));
        homvar(ii)=members(length(members));
        if homvar(ii)<=nvar
            error(['Sorry, I can only homogenize if homogenizing coordinate is not among the original variables.',...
                    '\nPlease check the specification of HomStruct.']);
        end
    end
    % loop thru equations
	jj=0;
	for ip=1:npol
        ii=jj+1; jj=jj+nummono(ip);
        mdegree = HomStruct*exponent(:,ii:jj);
        totmdegree = max([mdegree lpdmdeg(:,ip)],[],2);
        degrees(:,ip) = totmdegree;
        exponent(homvar,ii:jj)=totmdegree*ones(1,nummono(ip))-mdegree;
	end
end