% incrindx.m : function for incrementing index for startpoints
%   and (optionally) computing the next startpoint x
% Inputs: nvar, degrees, Astart, ptrs, epsstart,
%         index, level, B, P, Pstack
%     Also, expects ProjTransf as a global variable if x is requested
% Output: done, index, level, B, P, Pstack, x (optional)
%
% This is called only by lpdstart and is not intended for independent use
% nvar, degrees, Astart = the LPD structure 
%                          with random linear coefficients in Astart
% Each call to incrindx cycles to the next combination of linear factors
%  that can give a solution.  This is judged using epsstart to detect
%  pivots that are too small, i.e., combinations of factors that are
%  rank deficient and thus cannot give a start point.
% The rest of the inputs and outputs are keeping track of where we are
%  in the combinatorics, including keeping a triangularized version of
%  the partial set of linear factors.

function [done, index, level, B, P, Pstack, x]=...
   incrindx(nvar,degrees,Astart,ptrs,epsstart,index,level,B,P,Pstack)

done=0;
x=[];
% level>nvar means we just put out a soln. Back up 1 level and get the next.
if level>nvar,
   level=level-1;
end;
% increment to next factor at current level if possible
if (index(level)<degrees(level)),
 tryagain=1;
 while tryagain,
   index(level)=index(level)+1;
   % Insert the new factor into B matrix
   B(level,:)=Astart(ptrs(level)+index(level),:);
   %disp(sprintf('New factor: level=%d, index=%d, Arow=%d',level,index(level),ptrs(level)+index(level)))
   % Gaussian elimination
   if level>1, 
     for kk=1:level-1,
       B(level,[P(kk:nvar) nvar+1])=B(level,[P(kk:nvar) nvar+1]) ...
                           -(B(level,P(kk))/B(kk,P(kk)))*B(kk,[P(kk:nvar) nvar+1]);
     end;
   end;
   % Column pivoting
   Pstack(level,:)=P;
   [bmax,imax]=max(abs( (B(level,P(level:nvar)) )' ) );
   % if there is no nonzero pivot, go back and get another factor
   if bmax<epsstart,
      tryagain=index(level)<degrees(level); 
   else,
      tryagain=0; 
      imx=imax-1;
      istor=P(level); P(level)=P(level+imx); P(level+imx)=istor;
      % Advance to the next level
      level=level+1; 
      if level>nvar,
         % We've filled in all levels, so we have a solution
         % Compute it if the call has asked for it
         if nargout==7,
            x=B(:,1:nvar)\B(:,nvar+1);
         end;
         return;
      else,
         index(level)=0; 
      end;
   end;
 end;
% Current level all played out.  Back up 1 level, and try again.
elseif level>1,
   level=level-1;
   P=Pstack(level,:);
else,
   done=1;
end;
return;
   
