% LPDSOLVE
%
% Script to solve polynomial systems by continuation via Linear Product Decompostion
% Solve a polynomial system with a given linear product-decomposition.
% Uses linearly-interpolated homotopy.  Endgame uses fractional power
% series.
% 
% INPUTS: Assumes the following have been set 
%   global nvar degrees FFUN
%   nvar      = number of variables
%   degrees   = vector whose i-th element is the degree of equation i
%   HomStruct = (optional) mhom-by-(nvar+mhom) matrix of 1's and zeros
%          Element (i,j)=1 indicates variable j is in group i.
%          Each variable must be in exactly one group.
%          If missing or empty, default to 1-homogeneous in all variables.
%   homvar    = (optional) 1-by-mhom list identifying the homogenizing variable for
%          each group.  If missing or empty, assume last variable in the
%          group.
%   LPDstruct = linear product-decomposition structure, having 
%          sum(degrees) number of rows and nvar+mhom columns.
%          There are degrees(i) rows for equation i, each representing
%          a linear factor, having 1 in column j if variable j appears
%          in the factor and 0 otherwise.  
%   FFUN    = string containing name of m-file for function and Jacobian
%          of system.  It should be homogenized.
% OUTPUTS:
%   xsoln  = matrix of solutions, listed column-wise
%   stats  = i-th column is 
%            [t, errsize, fcn_err, cycle #, cond. #, # of fcn evals]
%            for solution i
%   xendgame = solution points at t=t_endgame>0, before endgame starts
%
% GLOBALS:
%   nvar degrees FFUN
%   <numerous,... see htopyset.m for the full list>
%
% Example:
%   To find solutions for the system using two continuation paths
%        x^2-1=0,  x*y-1=0
%   use the following script
%      global nvar degrees FFUN
%      nvar=2;
%      degrees=[2 2];
%      LPDstruct=[1 0 1; 1 0 1;   1 0 1; 0 1 1];
%      FFUN='simplfcn';
%      lpdsolve
%
%   where simplfcn.m is as follows:
%
% function [f,df]=simplfcn(x)
% f=[x(1)^2-x(3)^2; x(1)*x(2)-x(3)^2];
% df=[
% 2*x(1)   0   -2*x(3)
% x(2)   x(1)  -2*x(3)
% ];

% Load in homotopy control parameters
htopyset

% set the Initialization flag, used in lpdgfun
global Initialize
Initialize=1;

% Now, make a randomized start system 
% generate start constants based on structural matrix "LPDstruct"
global LPDstart
LPDstart=crand(size(LPDstruct)).*LPDstruct;

% If HomStruct is missing or empty, default to 1-homogeneous
if ~exist('HomStruct'), 
    warning('HomStruct missing. Will set to 1-homogeneous. Put ''HomStruct=[];'' in your script to avoid future warnings.')
    HomStruct = ones(1,nvar+1);
    homvar = nvar+1;
else
    if isempty(HomStruct)
        HomStruct = ones(1,nvar+1);
    end
end
% if homvar is not defined, default to last variable of each group
% if it is empty, assume we are on a projective space
if ~exist('homvar','var')
    mhom=size(HomStruct,1);
    for ii=1:mhom
        members=find(HomStruct(ii,:));
        homvar(ii) = members(length(members));
    end
end

% Set up projective transformation
% check the structure for validity
if size(LPDstruct,2)~=size(HomStruct,2),
    error('LPDsolve: LPDstruct and HomStruct must have same number of columns.');
end
if any(sum(HomStruct,1)~=1),
    error('LPDsolve: each variable must appear exactly once in HomStruct');
end
global ProjTransf  % pass to hfun and lpdstart
ProjTransf = crand(size(HomStruct)).*HomStruct;
mhom=size(HomStruct,1);

% do some consistency checking
if nvar~=size(degrees,2)
    error('LPDsolve: nvar must equal size(degrees,2).')
end
if sum(sum(degrees))~=size(LPDstart,1),
    error('LPDsolve: # of rows in LPDstruct must equal sum of all multi-degrees.')
end
% Solve the start system,
%  filtering out degenerate points, if filter exists
if exist('STARTPOINTFILTER')
    if exist(STARTPOINTFILTER)==2
        disp(['Applying startpoint filter: ' STARTPOINTFILTER])
        % Solve the start system
        startpoint=lpdstart(nvar,sum(degrees,1),LPDstart,homvar,STARTPOINTFILTER);
    else
        error(['Designated startpoint filter does not exist: ' STARTPOINTFILTER]);
    end
else
    disp(['No STARTPOINTFILTER defined. Will run all startpoints.'])
    startpoint=lpdstart(nvar,sum(degrees,1),LPDstart,homvar);
end
nstart=size(startpoint,2);
disp(['Number of startpoints = ' num2str(nstart)]);

%clean up
clear Initialize

% exit branch for calculating only the startpoints (used by bezno.m)
if exist('StartPointOnly','var')
    if StartPointOnly
        return
    end
end

% Set up homotopy
hfun='lpdhfun';
format compact; format short e;

% solve it
allownewpatch=1;  % this option generally helps
[xsoln,stats,xendgame,successprep,tprep]=endgamer(startpoint,hfun,allownewpatch);
clear allownewpatch