% lpdstart : Solve a linear product decomposition start system
%
% [startpoint]=lpdstart(nvar,degrees,LPDstart,homvar,STARTPOINTFILTER)
%
% INPUTS
%  nvar     = number of variables
%  degrees  = row vector of degrees of the equations
%  LPDstart = matrix of coefficients of linear factors (see lpdsolve)
%  homvar   = list of the homogenizing variables.  If not empty, we are
%     on a Eucliden space, and we use this list to ignore solutions at infinity.  
%     If empty, we work on projective space and ignore nothing.
%     Note, usually homvar has one entry per row in ProjTransf, but not
%     if some projective subspace is to be treated as native projective
%     (as opposed to being the homogenization of a Euclidean subspace).
%  STARTPOINTFILTER = string name of a filtering function (optional)
%     if present, feval(STARTPOINTFILTER,x) should return 1 for
%     start points to keep, and 0 for those to be discarded
%     See CstrFilt for example of working on (C*)^n, C* = C\setminus 0
% OUTPUTS
%  startpoint = matrix of startpoints, listed columnwise
%     These will satisfy the projective transformation in global ProjTransf
% GLOBALS
%  epsstart ProjTransf
%  where epsstart is used for deciding which combinations of linear factors
%    give valid solutions.

function [startpoint]=lpdstart(nvar,degrees,LPDstart,homvar,STARTPOINTFILTER)
global epsstart 

% Pre-pend the projective transformation
global ProjTransf
mhom=size(ProjTransf,1);
ddegrees = [ones(1,mhom) degrees];
AAstart = [ProjTransf, ones(mhom,1); LPDstart, zeros(size(LPDstart,1),1)];
nnvar = nvar+mhom;
%
% initialize pointers
ptrs=zeros(nnvar,1);
for ii=2:nnvar, ptrs(ii)=ptrs(ii-1)+ddegrees(ii-1); end;
%
% Initialization data
index=zeros(1,nnvar);  level=1; 
Pstack=[];  clear x;
B=zeros(nnvar,nnvar+1); P=[1:nnvar]; 
% compute start points 
done=0; path=0;
while ~done,
  % incrindx generates start points by tree search. It may take several calls
  %   to descend the tree
  [done, index, level, B, P, Pstack, x]=...
      incrindx(nnvar,ddegrees,AAstart,ptrs,epsstart,index,level,B,P,Pstack);
  % test 'level' to see if we have a start point
  if level>nnvar, 
    % we have a potential startpoint
    % check if any homogenizing coordinates are zero
    keep = ~any(abs(x(homvar))<epsstart);
    % check the filter
    if nargin>4, keep= keep & feval(STARTPOINTFILTER,x); end
    if keep,
        % use Newton to sharpen it
        [xx,success,condfx]=newton('lpdhfun',x,3,1e-12,1e-10,1); % use hfun at t=1 to invoke ProjTransf
        if ~success, disp(['Warning: startpoint number ' num2str(path) ' not sharp.']); end;
        if (condfx > 1/epsstart)
           disp(['  Startpoint ' num2str(path) ' rejected as singular.']);
        else
           % check the filter again on sharpened point
           if nargin>4, keep=feval(STARTPOINTFILTER,x); else, keep=1; end
           if keep
               path=path+1; 
               startpoint(:,path)=xx;
           end
        end
    end;
  end;
end;