% lpdtab
% A script for solving a system given in tableau format
% using a linear-product decomposition homotopy
%
% To execute this script, the workspace must contain the following
%   -> tableau    = polynomial system given in expanded form
%   -> LPDstruct  = description of linear-product structure
%   -> LPDdegrees = list of number of factors in each eqn of LPDstruct
% optionally
%   -> HomStruct  = multi-homogeneous structure
%        if missing or empty, defaults to 1-homogeneous
%
% Assume the tableau is in the workspace in matrix "tableau"
% which has
%   1) coefficients in first column
%   2) exponents of variables x1,...,xn in columns 2,...,(n+1)
%   3) end of polynomials marked with a -1 in column 2
%   4) we assume the system is NOT homogeneous, and we
%      multi-homogenize it
% LPDstruct,LPDdegrees = linear product-decomposition structure,
%          LPDstruct must have sum(LPDdegrees) number of rows and nvar+mhom columns.
%          There are LPDdegrees(i) rows for equation i, each representing
%          a linear factor, having 1 in column j if variable j appears
%          in the factor and 0 otherwise.
%    It is the user's responsibility to verify that the specified
%    LPDstruct is compatible with tableau and HomStruct
% The multi-homogeneous structure is specified in matrix HomStruct
% HomStruct =  mhom-by-(nvar+mhom) matrix of 1's and zeros
%          Element (i,j)=1 indicates variable j is in group i.
%          Each variable must be in exactly one group.
%          The homogenizing variables must all come at the end, that is,
%          the last mhom columns of HomStruct
if ~exist('tableau','var')
    error('Matrix ''tableau'' must be provided by user.')
end
if ~exist('LPDstruct','var')
    error('LPDstruct must be provided by user.')
end
% for missing HomStruct, set empty to default to 1-hom (total degree)
% (this will act the same as totdtab)
if ~exist('HomStruct','var')
    HomStruct=[];
end
% decode the tableau and homogenize it
global nvar degrees
nvar = size(tableau,2)-1;
global nummono coeflist exponent
ishom=0;
[npol,nvar,degrees,nummono,coeflist,exponent]=ftabprep(tableau,HomStruct,ishom,LPDstruct,LPDdegrees);
if npol~=nvar
    error(['number of variables =', num2str(nvar),', must be equal to number of polynomials =' num2str(npol),'.'])
end
clear homvar
degrees = sum(degrees,1);

% pass the function name via global FFUN
global FFUN
FFUN='ftabcall';

% solve it
lpdsolve
