function [x,errsize,success,nfe,fcnerr]=newton2(hfun,x,t,epsok,maxit,persist)
% [x,errsize,success,fcnerr]=newton2(hfun,x,t,epsok,maxit,persist)
%
% Newton's method to find root of hfun(x,t)=0
%  near start value of x for fixed value of t
% Quits when estimated error is less than epsok or if the
%  number of iterations exceeds maxit.
%
% INPUTS
%  hfun    = string name of homotopy function, h(x,t)=0
%  x       = initial estimate of solution
%  t       = t value at which we wish to solve for x
%  epsok   = accuracy target
%  maxit   = max number of Newton iterations allowed
%  persist == 1 means keep going even if error in x increases
%          == 0 means stop if error doubles or worse
% OUTPUTS
%  x       = final solution
%  errsize = size of last Newton step
%  success = 1, if converged within epsok
%  nfe     = number of function evaluations used
%  fcnerr  = magnitude of h(x,t) on return
iter=0;
fail=0;
errsize=inf;
while ~fail & errsize>epsok & iter<maxit,
    iter=iter+1;
	[h,dhx,dht]=feval(hfun,x,t);
	errx=-dhx\h;
	% The following tests the linear solution, which might
	% fail if dhx is highly singular.  It would be better if Matlab would tell
	% us this directly, but it just prints a warning.
	if max(abs(dhx*errx+h))<1e-8
       esize=max(abs(errx));
	else
       esize=inf;
       fail=1;
	end
    % if dhx not too singular, update
    if ~fail
	    x=x+errx;
    end
    olderr =errsize;
    errsize=esize;
    % check persistence flag & quit if appropriate
    if ~persist & errsize>2*olderr, break; end;
end
success= errsize<epsok;
nfe = iter;
if nargout>4, fcnerr = max(abs(h)); end
