% PARSOLVE
%
% Script to solve polynomial systems by parameter continuation
% A solution set for a generic case must already be loaded before PARSOLVE
% is launched.  Also, we need to know the system function and how
% it is homogenized.  A default to 1-homogeneous is available.
% 
% The user can set the path via PATHFUN. If does not exist or if
% it is empty, we will default to a linearly-interpolated path, 
% in which case the parameter space must be Euclidean.
%
% The endgame uses fractional power series.
% 
% INPUTS: Assumes the following have been set 
%   HomStruct = (optional) mhom-by-(nvar+mhom) matrix of 1's and zeros
%          Element (i,j)=1 indicates variable j is in group i.
%          Each variable must be in exactly one group.
%          If missing or empty, default to 1-homogeneous in all variables.
%   FFUN    = Global. String containing name of m-file for function and Jacobian
%          of system.  It should be homogenized.
%   ParStart = Global. Column of starting parameters
%   ParGoal  = Gloabl. Column of target parameters
%   xstart  = starting solutions at ParStart, listed column-wise.
%   PathFun = Global. (optional) String name of path function.
%             If missing or empty, default to 'lin_path'
% OUTPUTS:
%   xsoln  = matrix of solutions, listed column-wise
%   stats  = i-th column is 
%            [t, errsize, fcn_err, cycle #, cond. #, # of fcn evals]
%            for solution i
%   xendgame = solution points at t=t_endgame>0, before endgame starts
%
% GLOBALS:
%   FFUN ParStart ParGoal
%   <numerous,... see htopyset.m for the full list>
%

% Load in homotopy control parameters
htopyset

% check that everything needed is available
if exist('FFUN','var')
    if ~isglobal(FFUN)
        warning('FFUN should be global. I will make it global now.')
        global FFUN
    end
else
    error('No FFUN defined. Exiting.')
end
if exist('ParStart','var')
    if ~isglobal(ParStart)
        warning('ParStart should be global. I will make it global now.')
        global ParStart
    end
else
    error('No ParStart defined. Exiting.')
end
if ~exist('xstart','var')
    error('No xstart defined. Exiting.')
end
if exist('ParGoal','var')
    if ~isglobal(ParGoal)
        warning('ParGoal should be global. I will make it global now.')
        global ParGoal
    end
else
    error('No ParGoal defined. Exiting.')
end

% provide defaults for PathFun & HomStruct
if exist('PathFun','var')
    if ~isglobal(PathFun)
        warning('PathFun should be global. I will make it global now.')
        global PathFun
    end
else
    disp('No PathFun defined. Defaulting to a linear path.')
    global PathFun
    PathFun = 'lin_path';
end
% determine # of variables from size of xstart
nvars = size(xstart,1);
if exist('HomStruct','var')
    if isempty(HomStruct)
        HomStruct = ones(1,nvars); % default to 1-hom
    else
        if nvars~=size(HomStruct,2)
            error('HomStruct columns must be equal in number to rows of xstart.')
        end
    end
else
    HomStruct = ones(1,nvars); 
end
global ProjTransf
ProjTransf = crand(size(HomStruct)).*(HomStruct>0);

% Set up homotopy
hfun='parhfun';

% run the homotopy
[xsoln,stats,xendgame,successprep,tprep]=endgamer(xstart,hfun,1);
