function [pairs,dist,tol,nNaN]=pathcros(xsoln,HomStruct)
% [pairs,dist,tol,nNaN]=pathcros(xsoln,HomStruct) : check for path crossings
%   i.e., check for columns of xsoln that match
% Each entry in pair(1,:) has a match that is in pair(2,:).
% Double roots appear twice, triples appear thrice, etc.
%
% Assumes xsoln is in projective space, so normalize
% largest (magnitude) coordinate to 1 before comparing
%
% The tolerance for comparison is set interactively by user 
% by clicking mouse on a histogram.
%
% Input:
%   xsoln = matrix of points in projective space, listed columnwise
%   HomStruct = homogeneous groupings. 
%         if missing or empty, default to 1-homogeneous.
% Output:
%   pairs = 2 x nmatch, 
%           each column gives the indices in xsoln of a matching pair
%   dist  = 1 x nmatch, max(abs( difference of the pair ))
%   tol   = log10(tolerance for matching)
%   nNaN  = number of points in xsoln that contain a NaN, i.e., failed
%             during the continuation run.
n=size(xsoln,1);
[y,iNaN] = find(isnan(xsoln(1,:)));
nNaN = length(iNaN);
if nNaN, disp(['Warning: there are ' num2str(nNaN) ' solutions containing NaN']); end
xsoln(:,iNaN)=zeros(n,nNaN);
if nargin<2
    HomStruct=ones(1,n);
elseif isempty(HomStruct)
    HomStruct=ones(1,n);
end
% loop through, normalizing each group
y=zeros(size(xsoln));
for ii=1:size(HomStruct,1)
    members=find(HomStruct(ii,:));
	w = max(abs(xsoln(members,:)));
    w(iNaN) = 1;
	y(members,:) = xsoln(members,:) * diag(1./w);
end
% find nearest points in two random directions
v=crand(2,n);
test=real(v*y);
ny = size(y,2);
near=zeros(4,ny);
for ii=1:2
	[s,k]=sort(test(ii,:));
	k=[0 k 0];
    irow=2*ii-1;
    for jj=2:(ny+1)
        near([irow irow+1],k(jj))=[k(jj-1); k(jj+1)];
    end
end
% compute distance to the 4 neighbors and find the closest one
dd=zeros(4,ny);
for ii=1:ny
    for jj=1:4
        if near(jj,ii)==0
            dd(jj,ii)=inf;
        else
            dd(jj,ii)=max(abs(y(:,ii)-y(:,near(jj,ii))));
        end
    end
end
[d,k]=min(dd);
% plot histogram of distances to help user pick a tolerance
edges=[-inf, -12:1:0, inf];
e2   =[-13:1];
nhist = histc(log10(d+1e-16),edges);
bar(e2,nhist,'histc');
title('Histogram of solution differences')
xlabel('Log10(difference)')
ylabel('# of solutions')
disp('Click mouse to indicate path crossing tolerance.')
[tol,yclick]=ginput(1);
% find the matching pairs and report on them
match= find(log10(d+1e-20)<tol);
pairs(1,:) = match; 
for ii=1:length(match)
    pairs(2,ii)= near(k(match(ii)),match(ii));
end
dist=d(match);
return