function [tabout,varscale,eqnscale]=scalepol(tableau)
% function [tabout,varscale,eqnscale]=scalepol(tableau)
% Rescale a polynomial system given in tableau form
% Input:
%   tableau   = tableau matrix defining the system, with
%     1) coefficients in first column
%     2) exponents of variables x1,...,xn in columns 2,...,(n+1)
%     3) end of polynomials marked with a -1 in column 2
% Output:
%  tabout   = tableau with scaled coefficients
%  varscale = old variables are varscale.*newvariables
%  eqnscale = new equations are eqnscale.*oldequations
% Method:
%  minimize sum of squares of logs of the new coefficients

% parse tableau into its individual polynomials
nvar = size(tableau,2)-1;
monos = tableau(:,2)>=0; % end of polynomial marked by negative exponent on x1
npol = nnz(~monos);
keep = find(monos);
coeflist = tableau(keep,1);
exponent = tableau(keep,2:(nvar+1))';
clear keep
% check that exponents are nonnegative integers
if any(any(floor(exponent)-exponent))
    error('All exponents must be integer')
end
if any(any(exponent<0))
    error('All exponents must be non-negative')
end
% count the monomials
eop = [0 find(~monos)']; % list of indices for end of polynomials
nummono = eop(2:(npol+1))-eop(1:npol)-1; % count the monomials

% form scaling problem as linear least-squares function of logs
% columns of A correspond to scaling factors of the variables followed
%  by scaling factors for the equations
% A has one row for each term (to minimize the magnitude of coefficients)
% AA has one row for each pair of terms (within equations) (to minimize
%   variation in magnitude of the coefficients)
nterm = sum(nummono);
A=zeros(nterm,npol+nvar); b=zeros(nterm,1);
npair = sum( nummono.*(nummono-1) )/2;
AA=zeros(npair,npol+nvar); bb=zeros(npair,1);
nterm=0; npair=0;
for ieq=1:npol
    nt=nummono(ieq);
    % log magnitude of coefficients
    A(nterm+[1:nt],[[1:nvar] nvar+ieq]) = [exponent(:,nterm+[1:nt])' ones(nt,1)];
    b(nterm+[1:nt]) = log(abs(coeflist(nterm+[1:nt])));
    for ii=1:nt-1
        for jj=(ii+1):nt
            npair = npair+1;
            AA(npair,[[1:nvar] nvar+ieq]) = [exponent(:,nterm+ii)'-exponent(:,nterm+jj)' 0];
            bb(npair) = log(abs(coeflist(nterm+ii))) - log(abs(coeflist(nterm+jj)));
        end
    end
    nterm = nterm+nt;
end

% solve least-squares problem
alpha = -([A;AA]\[b;bb]);

% load results into output variables
varscale = exp(alpha(1:nvar));
eqnscale = exp(alpha(nvar+[1:npol]));
tabout=tableau;
termscale = exp(A*alpha);
nterm=0; k=0;
for ieq=1:npol
    nt=nummono(ieq);
    tabout(k+[1:nt],1) = coeflist(nterm+[1:nt]).*termscale(nterm+[1:nt]);
    nterm=nterm+nt;
    k=k+nt+1;
end
