function [x,t,success,stepsize,errsize,nfe]= ...
            tracker(hfun,x,tangent,t,stepstart,tgoal,epstrack,cycle,maxnfe)
% TRACKER: variable-step predictor-corrector path tracker 
%          with Euler predictor modified by cycle number
%          and Newton-Raphson corrector
%          tracks x from t to tgoal, with t>tgoal
%
% function [x,t,success,stepsize,errsize,nfe]= ...
%            tracker(hfun,x,tangent,t,stepstart,tgoal,epstrack,cycle,maxnfe)
%
%
% INPUTS
%   hfun      = string name of homotopy function h(x,t)
%                evaluated as [h,hx,ht]=feval(hfun,x,t)
%   x         = starting value of x. An approximate solution of h(x,t)=0
%   tangent   = tangent at (x,t). If empty, is computed fresh.  
%               If known ahead, passing it in saves a bit of computation
%   t         = t value at start
%   stepstart = initial step size to try
%   tgoal     = stopping value for t.  We assume tgoal<t.
%   epstrack  = path tracking accuracy to be maintained
%   cycle     = estimated cycle number. Use 1 except in endgame.
%   maxnfe    = max number of function evaluations to use
% OUTPUTS
%   x,t       = final solution estimate. If successful, t=tgoal
%   success   = 1, if successful; 0, if not
%   stepsize  = final step size in t. Can be used as stepstart if we call
%                again for progress along the same path
%   errsize   = last Newton residual
%   nfe       = number of function evaluations used
%
% GLOBALS
%   stepmin maxit epstiny epssample

% Oct. 8, 2003: changed stepsize to mean step in t (was step in arclength of x)
%       Also, cleaned up using calls to newton2

global stepmin maxit epstiny
global epssample
global verbose

goodstep=0; success=1; nfe=0;
stepsize=stepstart;
iter=0;

% if tangent is passed in as empty, compute it now.
if isempty(tangent),
   [h,dhx,dht]=feval(hfun,x,t);
   tangent = -dhx\dht;
   nfe=nfe+1;
end

% tracking loop.
while t>tgoal,
   % detect zero-length tangent error (should not happen for correct
   % homotopies, but check just in case)
   tanlen=sum(abs(tangent));
   if tanlen<epstiny, 
      disp(['Path stalled at t=' num2str(t)])
      error('Tangent vector is zero.')
   end;
   % Step t.  Do not step past the goal.
   step=min(t-tgoal,stepsize);
   tnew=t-step; 
   % PREDICT 
   %  linear (Euler) step with modification for cycle numbers>1
   r = t*cycle*( (tnew/t)^(1/cycle)-1. ); % for c=1, r = tnew-t
   xnew = x + tangent*r;
   % CORRECT
   [xnew,errsize,success,nfe2]=newton2(hfun,xnew,tnew,epstrack,maxit,1);
   nfe=nfe+nfe2;
   % STEP LENGTH ADJUSTMENT
   if success 
      % step was good.  Update
      x=xnew; t=tnew;
      [h,dhx,dht]=feval(hfun,x,t); tangent=-dhx\dht;
      % If 5 successive good ones, double stepsize.
      goodstep=goodstep+1; 
      if goodstep==5, stepsize=2*stepsize; goodstep=0; end;
   else,
      % step failed.
      goodstep=0;
      % cut stepsize
      stepsize=step/2.;
      % check if we should quit
      success = stepsize>stepmin & nfe<maxnfe;
      if ~success, 
         if verbose
             disp(sprintf('Tracker failed: ')); t, stepsize
         end
         return;  % return with last point on path and success=0
      end;
   end;
%trac;
end;
% ENDPOINT REFINEMENT
% We've reached the goal t, refine the answer with Newton
[x,errsize,ss,nfe2] = newton2(hfun,x,tgoal,epssample,10,0);
nfe = nfe+nfe2;
