function [Wsuper,Lslice,Stats,Nonsoln,NonsolnStats,Degen,DegenStats,Bad,BadStats] = ...
    witsup(fsysname,NSYSpol,NSYSvar,SYSdegrees,eps0,eps_inf,topdimen)
% [Wsuper,Lslice,Stats,Nonsoln,NonsolnStats,Degen,DegenStats,Bad,BadStats] = ...
%          witsup(fsysname,NSYSpol,NSYSvar,SYSdegrees,eps0,epsinf);
% Compute witness supersets
% Algorithm from Chapter 13 of Sommese & Wampler based on randomization and
% slicing.  Use intrinsic formulation of slicing planes.
%
% System is assumed to be 1-homogenized, with homogeneous coordinate in last
%    position.
% System is assumed to be nontrivial, so we don't test for that.
%
% INPUT:
%  fsysname = string name of function for system of polynomials
%      Will be called using [f,df]=feval(fsysname,x)
%  NSYSpol = # of polynomials in the system
%  NSYSvar = # of variables
%  SYSdegrees = row matrix list of the degrees of the polynomials
%  eps0    = tolerance value. Solutions have |f|<eps0
%  eps_inf = tolerance for declaring solution at infinity
%  topdimen= (optional) starting top dimension.  defaults to codimension 1
%
% OUTPUT:
%  all outputs are cells arrays, indexed by codimension 
%  Wsuper{i}  = witness superset for codimension i. 
%               points given in homogeneous coordinates.
%  Lslice{i}  = slice, represented intrinsically as (NSYSvar+1) x (i+1)
%                 matrix, where "+1" is for homogeneous coordinate
%  Stats{i}   = stats matrix for the solutions in Wsuper. 
%                 Usual format from endgamer with function error (original
%                 system) appended as last row.
%  Nonsoln{i} = nonsolutions from i-th slice. eps0 is tolerance
%  NonsolnStats{i} = stats for the nonsolutions.
%  Degen{i}   = endpoints at infinity for i-th slice, eps_inf is tolerance
%  DegenStats{i}= their stats
%  Bad{i}     = endpoints that didn't converge well
%  BadStats{i}= their stats
%
% The information for nonsolutions is returned to give the opportunity of
% checking them for quality.
%  
% This routine calls witsupi for each dimension N-1,...,N-rank f

if nargin<7, 
    codimen0 = 1;
else
    codimen0 = NSYSvar-topdimen;
end

% get rank of the system
rnk = ranksys(fsysname,NSYSvar); 

% ii is co-dimension
for ii=codimen0:rnk
    idimen = NSYSvar-ii;
    [Wsuper{ii},Lslice{ii},Stats{ii},Nonsoln{ii},NonsolnStats{ii},Degen{ii},DegenStats{ii},Bad{ii},BadStats{ii}]= ...
        witsupi(fsysname,NSYSpol,NSYSvar,SYSdegrees,idimen,eps0,eps_inf);
end
